# Script for 1D PME using Fourier Galerkin:

import sys
sys.path.insert(0,'..')

import numpy as np
import numpy.fft as fft
import fourier
import oscintmodule as oscint

import matplotlib.pyplot as plt
import matplotlib.animation as animation
#######################################################
#               PARAMETERS:
#######################################################
N = 128

# Initial data:
def initial_data(x):
    return np.sign(np.pi-x)
initial_data = np.vectorize(initial_data)

# Nonlinearity:
def phi(u):
    return u*(u>0)

phi = np.vectorize(phi)


# Fractional laplacian:
alpha = 1.5
eps = 1E-8
c_alpha = fourier.get_big_c_alpha(alpha, eps)

# End time:
T = 1.0

# Delta time:
dt = 1E-4

#SVV parameter:
theta = 0.5

# Get SVV data:
epsN = 0.05*fourier.get_svv_eps(theta,N)
mN = fourier.get_svv_m(theta,N)
qhat = fourier.get_svv_qhat(mN,N)
#qhat = np.zeros(N/2+1)
######################################################
#Get spatial domain:
x = 2*np.pi/N*np.arange(N)

#Initialise data:
u = initial_data(x)
u = fourier.fejer_approximation(u,N)
uc = fourier.physical_to_fourier(u,N)

minu = min(u)
maxu = max(u)
######################################################
#               ANIMATION:
###################################################### 
# Wanted length of animation (in secs):
ani_T = 10.

# Frame rate:
fps = 30

#Number of frames:
num_frames = int(fps*ani_T)

# Number of time steps per frame:
time_steps = int(T/(fps*dt*ani_T))+1


#Start figure:
fig = plt.figure()
ax = fig.add_subplot(111, autoscale_on=False,
            xlim=(0.,2*np.pi), ylim=(-0.1+minu,0.1+maxu))

line, = ax.plot([],[], 'b-', lw=1.5)

time_text = ax.text(0.02, 0.95,'', transform=ax.transAxes)

# Initialise time:
t = 0.

# Animation functions:
def init():
    """Initialise animation"""
    line.set_data([],[])
    time_text.set_text('')
    return line, time_text

def animate(i):
    """Perform animation step"""
    global u, x, dt, N, qhat, epsN, t, time_steps

    for k in xrange(time_steps):
        t += dt

        u = fourier.fourier_svv_RK4(u,N,phi,alpha, c_alpha, qhat,dt,epsN)

    line.set_data(x,u)
    time_text.set_text('time= %0.4f' %t)
    return line, time_text
    
# Let's try to animate:
from time import time
t0 = time()
animate(0)
t1 = time()
interval = 1000*dt - (t1-t0)

ani = animation.FuncAnimation(fig, animate, frames=num_frames,
                    interval=interval, blit=True, init_func=init)

mywriter = animation.AVConvWriter(fps=fps)
ani.save('example_animation.mp4', writer=mywriter,
        extra_args=['-vcodec', 'libx264'])


