# Script for solving fractional porous medium equations.
# Plots solution at end time together with initial data.

# Import section:
import sys
sys.path.insert(0,'..')

import numpy as np
import numpy.fft as fft
import fourier

import matplotlib.pyplot as plt

################################
#       Parameters:
################################

N = 128

# Initial data:
def initial_data(x):
    return np.sign(np.pi - x)

initial_data = np.vectorize(initial_data)

# Nonlinearity
def phi(u):
    return u*(u>0)

# Fractional laplacian:
alpha = 1.5
eps = 1E-8
c_alpha = fourier.get_big_c_alpha(alpha,eps)

# End time:
T = 2.

# Time step:
dt = 1E-4

# SVV parameters:
theta = 0.5

epsN = 0.05*fourier.get_svv_eps(theta,N)
mN = fourier.get_svv_m(theta, N)
qhat = fourier.get_svv_qhat(mN, N)

##########################################
#              Solving:
##########################################

# Get spatial domain:
x = 2.*np.pi/N*np.arange(N)

# Initialize u:
u = initial_data(x)

# Approximate the initial data?
#u = fourier.vallee_poussin_approximation(u,N)
#u = fourier.fejer_approximation(u,N)

# Initialize figure:
fig = plt.figure()
ax = fig.add_subplot(111, autoscale_on=False,
        xlim=(0,2.*np.pi), ylim=(-0.1+min(u), 0.5+max(u)))

# Plot initial data:
ax.plot(x,u, 'r--', lw=1.5, label='Initial data')

# Initialize time:
t = 0.

# Solve:
while (t<T):
    u = fourier.fourier_svv_RK4(u,N,phi,alpha,c_alpha,qhat,dt,epsN)
    t += dt

# plot solution:
ax.plot(x,u, 'b', lw=1.5, label='Numerical solution')
ax.set_xlabel('$x$')
ax.set_ylabel('$u$')

# Uncomment if you want to save figure.
#fig.savefig('example_figure.png')
plt.show()
