# Script for managing the various transformations in Fourier-
# physical space.
import numpy as np
import numpy.fft as fft
import oscintmodule as oscint
from scipy.special import gamma
from scipy.misc import factorial

def physical_to_fourier(u_points,N):
    """Function for getting Fourier coefficients
    from point values on the interval (0,2*Pi).
    """
    return fft.rfft(u_points,N)/N


def fourier_to_physical(u_fcoeff, N):
    """Getting back to point values from Fourier spectrum.
    """
    return N*fft.irfft(u_fcoeff, N)

def nonlinear_coeffs(u_points,N, phi):
    """Get the coefficients of Phi(u) given
    u_points.
    """
    return fft.rfft(phi(u_points),N)/N

def nonlinear_coeffs_from_fourier(u_fcoeff,N,phi):
    """Get the coefficients of Phi(u) given the
    fourier coefficients of u.
    """
    u_points = fourier_to_physical(u_fcoeff,N)
    return nonlinear_coeffs(u_points,N,phi)

###########################################
#          RUNGE-KUTTA 4th-ORDER:
###########################################
def get_time_derivative(uc, N, phi, qhat, xisq, xial, epsN, c_alpha):
    """Function for calculating the time derivative
    """
    phic = nonlinear_coeffs_from_fourier(uc,N,phi)
    return -c_alpha*xial*phic - epsN*xisq*qhat*uc

def fourier_svv_RK4(u_points,N,phi, alpha, c_alpha, qhat,dt, epsN):
    """Do a time step of length dt using
    the Runge-Kutta 4th order scheme.
    """
    xisq = np.arange(N/2+1)**2
    xial = np.arange(N/2+1)**(alpha)
    # First getting the coeffs:
    uc = physical_to_fourier(u_points,N)

    #First step:
    k1 = get_time_derivative(uc,N,phi,qhat,xisq, xial, epsN, c_alpha)

    #Temps:
    uc_temp = uc+0.5*dt*k1

    #Second Step:
    k2 = get_time_derivative(uc+0.5*dt*k1,N,phi,qhat,xisq, xial, epsN, c_alpha)

    #Third step:
    k3 = get_time_derivative(uc+0.5*dt*k2,N,phi,qhat,xisq, xial, epsN, c_alpha)

    #Fourth step:
    k4 = get_time_derivative(uc+dt*k3,N,phi,qhat,xisq, xial ,epsN, c_alpha)

    #Total step in coefficients:
    uc_new = uc + dt/6.*(k1+2*k2+2*k3+k4)

    #Return new nodal values:
    return fourier_to_physical(uc_new,N)

##########################################
#           SVV
#########################################
def get_svv_eps(theta,N):
    """Get the eps_N parameter for the
    spectral vanishing viscosity.
    """
    return N**(-theta)

def get_svv_m(theta,N):
    """Get the m_N parameter for the
    spectral vanishing viscosity.
    """
    return int(N**(0.5*theta)*np.log2(N)**(-0.5))+1

def get_svv_qhat(m,N):
    """Get the Fourier coefficients of the
    spectral vanishing viscosity kernel.
    """
    qhat = np.zeros(N/2+1)

    for i in range(m,2*m):
        qhat[i] = (i-m)/float(m)

    qhat[2*m:]=1.
    return qhat


##################################################
#       Get fractional laplacian coefficients:
##################################################
def get_little_c_alpha(alpha):
    if ((alpha <= 0) or (alpha >= 2)):
        print "Value for alpha is NOT valid."
        return -1
    
    return alpha*gamma((1+alpha)/2.)/(2*np.pi**(0.5+alpha)*gamma(1-alpha/2))

def get_theta_alpha(alpha, eps):
    """Get value for oscillatory integral for the fractional laplacian given alpha and
    error tolerance eps."""
    if ( (alpha <1) and (alpha > 0)):
        return gamma(1-alpha)*np.sin(np.pi/2.*(1-alpha))
    elif (alpha==1.):
        return np.pi/2.
    elif ( (alpha >1) and (alpha<2)):
        I1 = get_boundary_layer_integral(alpha,1.,eps/2.)
        I2 = oscint.oscillatoryQuadrature(1.,alpha,1.,eps/2.)
        return I1+I2
    else:
        print "Nonvalid value for alpha!!"
        return -1.

def get_big_c_alpha(alpha,eps):
    if( (alpha<=0) or (alpha>=2)):
        print "Value for alpha is NOT valid."
        return -1

    c = get_little_c_alpha(alpha)
    theta = get_theta_alpha(alpha,eps)

    return 2*c/alpha*theta

def get_boundary_layer_integral(alpha, b, eps):
    """Get the first part of the oscillatory integral."""
    #Find how many terms are neeeded:
    N=1
    Nbound = 1E6
    sufficientN = False
    while not sufficientN and (N<Nbound):
        sufficientN = (b**(2*N+2-alpha)/((2*N+2-alpha)*factorial(2*N+1))<eps)
        if not sufficientN:
            N += 1

    #Sum up the integrals:
    I = 0.
    for n in range(N+1):
        I += (-1.)**n*b**(2*n+2-alpha)/((2*n+2-alpha)*factorial(2*n+1))

    return I

####################################################
#           HANDLING INITIAL DATA:
####################################################
def fejer_approximation(u_points, N):
    """Given the point values u_points,
    returns the fejer approximation at the
    same points.
    """
    fejer_coeffs = (1. - (np.arange(N/2+1))/float(N/2))*physical_to_fourier(u_points,N)

    return fourier_to_physical(fejer_coeffs, N)


def vallee_poussin_approximation(u_points, N):
    """Given the point values u_points,
    returns the de la Vallee Poussin approximation
    at the same points.
    """
    filter_coeffs = np.zeros(N/2+1)
    filter_coeffs[:N/4] = 1.
    filter_coeffs[N/4:] = 2. - np.arange(N/4,N/2+1)/float(N/4)
    filter_coeffs = filter_coeffs*physical_to_fourier(u_points,N)

    return fourier_to_physical(filter_coeffs, N)

