#include <Python.h>
#include <math.h>

/*
 * Oscillatory function:
 */
double f(double x, double alpha)
{
    return pow(x,-alpha)*sin(x);
}
/*
 *         Provided C-function:
 */
void intdeo(double a, double alpha, double omega, double eps, 
    double *i, double *err)
{
    /* ---- adjustable parameter ---- */
    int mmax = 256, lmax = 5;
    double efs = 0.1, enoff = 0.40, pqoff = 2.9, ppoff = -0.72;
    /* ------------------------------ */
    int n, m, l, k;
    double pi4, epsln, epsh, frq4, per2, pp, pq, ehp, ehm, ir, h, iback, 
        irback, t, ep, em, tk, xw, wg, xa, fp, fm, errh, tn, errd;
    
    pi4 = atan(1.0);
    epsln = 1 - log(efs * eps);
    epsh = sqrt(efs * eps);
    n = (int) (enoff * epsln);
    frq4 = fabs(omega) / (2 * pi4);
    per2 = 4 * pi4 / fabs(omega);
    pq = pqoff / epsln;
    pp = ppoff - log(pq * pq * frq4);
    ehp = exp(2 * pq);
    ehm = 1 / ehp;
    xw = exp(pp - 2 * pi4);
    *i = f(a + sqrt(xw * (per2 * 0.5)),alpha);
    ir = *i * xw;
    *i *= per2 * 0.5;
    *err = fabs(*i);
    errh = *err;
    h = 2;
    m = 1;
    do {
        iback = *i;
        irback = ir;
        t = h * 0.5;
        do {
            em = exp(2 * pq * t);
            ep = pi4 * em;
            em = pi4 / em;
            tk = t;
            do {
                xw = exp(pp - ep - em);
                wg = sqrt(frq4 * xw + tk * tk);
                xa = xw / (tk + wg);
                wg = (pq * xw * (ep - em) + xa) / wg;
                fm = f(a + xa, alpha);
                fp = f(a + xa + per2 * tk,alpha);
                ir += (fp + fm) * xw;
                fm *= wg;
                fp *= per2 - wg;
                *i += fp + fm;
                if (m == 1) *err += fabs(fp) + fabs(fm);
                ep *= ehp;
                em *= ehm;
                tk += 1;
            } while (ep < epsln);
            if (m == 1) {
                errh = *err * epsh;
                *err *= eps;
            }
            tn = tk;
            while (fabs(fm) > *err) {
                xw = exp(pp - ep - em);
                xa = xw / tk * 0.5;
                wg = xa * (1 / tk + 2 * pq * (ep - em));
                fm = f(a + xa, alpha);
                ir += fm * xw;
                fm *= wg;
                *i += fm;
                ep *= ehp;
                em *= ehm;
                tk += 1;
            }
            fm = f(a + per2 * tn,alpha);
            em = per2 * fm;
            *i += em;
            if (fabs(fp) > *err || fabs(em) > *err) {
                l = 0;
                for (;;) {
                    l++;
                    tn += n;
                    em = fm;
                    fm = f(a + per2 * tn,alpha);
                    xa = fm;
                    ep = fm;
                    em += fm;
                    xw = 1;
                    wg = 1;
                    for (k = 1; k <= n - 1; k++) {
                        xw = xw * (n + 1 - k) / k;
                        wg += xw;
                        fp = f(a + per2 * (tn - k), alpha);
                        xa += fp;
                        ep += fp * wg;
                        em += fp * xw;
                    }
                    wg = per2 * n / (wg * n + xw);
                    em = wg * fabs(em);
                    if (em <= *err || l >= lmax) break;
                    *i += per2 * xa;
                }
                *i += wg * ep;
                if (em > *err) *err = em;
            }
            t += h;
        } while (t < 1);
        if (m == 1) {
            errd = 1 + 2 * errh;
        } else {
            errd = h * (fabs(*i - 2 * iback) + pq * fabs(ir - 2 * irback));
        }
        h *= 0.5;
        m *= 2;
    } while (errd > errh && m < mmax);
    *i *= h;
    if (errd > errh) {
        *err = -errd;
    } else {
        *err *= m * 0.5;
    }
}

/*
 * Function to be called from Python
 */
static PyObject* py_oscillatoryQuadrature(PyObject* self, PyObject* args)
{
    const double b, alpha, omega, eps;
    double i=0, err=256;

    /* Parse arguments from Python */
    if(!PyArg_ParseTuple(args, "dddd", &b, &alpha, &omega, &eps))
        return NULL;

    /* Evaluate integral using C-function */
    intdeo(b, alpha, omega, eps, &i, &err);

    /* MAke into Python type and return */
    return Py_BuildValue("d", i);
}


/*
 *  Set up methods table:
 */
static PyMethodDef oscintmodule_methods[] = {
    {"oscillatoryQuadrature", py_oscillatoryQuadrature, METH_VARARGS, "Function for evaluating oscillating integral"},
    {NULL, NULL, 0, NULL}
};

/*
 * Python calls this to let us initialize our module
 */
PyMODINIT_FUNC initoscintmodule(void)
{
    (void) Py_InitModule("oscintmodule", oscintmodule_methods);
}


